{{

______________________________________________________________________________
********************************************************* File starts here ***
Zerocat Chipflasher --- Flash free firmware, kick the Management Engine.

Copyright (C) 2020, 2021, 2022  Kai Mertens <kmx@posteo.net>

File pins.spin --- Define Propeller Pins
Compare to: firmware/src/libkick/proppins.h

This file is part of the Zerocat Chipflasher.

See end of file for terms of use.

******************************************************************************
}}


{{

# Object Configuration (set via Makefile)

* Tell us which SPI Clock Driver Strength to use:

    100%, 75%, 50%, 25% or according to DIP Switch (default)

* Tell us which SPI mode to use:

    Mode 0, Mode 3, or according to DIP Switch (default)

* Tell us which RS232 Baudrate to use:

    115200, 57600, 38400 or according to DIP Switch (default)

* Tell us which SPI Bus Access Time to use:

    Continuous Access, Short Access, or according to DIP Switch (default)

* Tell us which SPI Power-up Type to use:

    One Shot, Repetitive, or according to DIP Switch (default)

}}
'
'
' Maintain the configuration token:
'
' * enable : "^# \t "
' * disable: "^'~ #"
'
'
'~ #   define _CFG_DRV100
'~ #   define _CFG_DRV75
'~ #   define _CFG_DRV50
'~ #   define _CFG_DRV25
# 	   define _CFG_DRV_DIPSW
'
'
#   ifdef _CFG_DRV100
#       warning SPI Clock Driver: 100%
#   endif
#   ifdef _CFG_DRV75
#       warning SPI Clock Driver: 75%
#   endif
#   ifdef _CFG_DRV50
#       warning SPI Clock Driver: 50%
#   endif
#   ifdef _CFG_DRV25
#       warning SPI Clock Driver: 25%
#   endif
#   ifdef _CFG_DRV_DIPSW
#       warning SPI Clock Driver, Board v1: 50%
#       warning SPI Clock Driver, Board v2: According to DIP Switch
#   endif
'
'
'~ #   define _CFG_SPIMODE0
'~ #   define _CFG_SPIMODE3
# 	   define _CFG_SPIMODE_DIPSW
'
'
#   ifdef _CFG_SPIMODE0
#       warning SPI Mode: 0
#   endif
#   ifdef _CFG_SPIMODE3
#       warning SPI Mode: 3
#   endif
#   ifdef _CFG_SPIMODE_DIPSW
#       warning SPI Mode, Board v1: Mode 0
#       warning SPI Mode, Board v2: According to DIP Switch
#   endif
'
'
'~ #   define _CFG_BAUDRATE_38400
'~ #   define _CFG_BAUDRATE_57600
# 	   define _CFG_BAUDRATE_115200
'
'
#   ifdef _CFG_BAUDRATE_38400
#       warning RS232 Baudrate: 38400
#   endif
#   ifdef _CFG_BAUDRATE_57600
#       warning RS232 Baudrate, Board v1: 57600
#       warning RS232 Baudrate, Board v2: 57600 | According to DIP Switch
#   endif
#   ifdef _CFG_BAUDRATE_115200
#       warning RS232 Baudrate, Board v1: 115200
#       warning RS232 Baudrate, Board v2: 115200 | According to DIP Switch
#   endif
'
'
'~ #   define _CFG_SUSPEND_ALLOW
'~ #   define _CFG_SUSPEND_INHIBIT
# 	   define _CFG_SUSPEND_DIPSW
'
'
#   ifdef _CFG_SUSPEND_ALLOW
#       warning SPI Power-up Time: Normal
#   endif
#   ifdef _CFG_SUSPEND_INHIBIT
#       warning SPI Power-up Time: Extended
#   endif
#   ifdef _CFG_SUSPEND_DIPSW
#       warning SPI Power-up Time, Board v1: Extended
#       warning SPI Power-up Time, Board v2: According to DIP Switch
#   endif
'
'
'
'
'~ #   define _CFG_PWRUP_ONCE
'~ #   define _CFG_PWRUP_REPETITIVE
# 	   define _CFG_PWRUP_DIPSW
'
'
#   ifdef _CFG_PWRUP_ONCE
#       warning SPI Power-up Pulse: One shot
#   endif
#   ifdef _CFG_PWRUP_REPETITVE
#       warning SPI Power-up Pulse: Repetitive
#   endif
#   ifdef _CFG_PWRUP_DIPSW
#       warning SPI Power-up Pulse, Board v1: Repetitive
#       warning SPI Power-up Pulse, Board v2: According to DIP Switch
#   endif
'
'


{{


# Object Summary
}}
OBJ     'This directive triggers the compiler to insert a summary.


CON
{{


# Constants

}}
        ''Fast Addressable Pins 0..8
        PIN_CEn1        = 0                       'Chip Enable Pin #1, active low
        PIN_CEn0        = 1                       'Chip Enable Pin #0, active low
        PIN_MOSI        = 2                       'Propeller’s shift-out pin
        PIN_WPn         = 3                       'Write Protect Pin, active low
        PIN_SCLK3       = 4                       'SPI Clock Pin #3
        PIN_SCLK2       = 5                       'SPI Clock Pin #2
        PIN_SCLK1       = 6                       'SPI Clock Pin #1
        PIN_SCLK0       = 7                       'SPI Clock Pin #0
        PIN_MISO        = 8                       'Propeller’s shift-in pin
        ''


        ''Slow Addressable Pins 9..31
        PIN_PLUGTESTn   = 9                       'This pin is used detect an adapter plug
        PIN_PNP         = 10                      'Disable SPI bus power through PNP MOSFET, active high
        PIN_HOLDn       = 12                      'Hold Pin, active low
        PIN_D3          = 13                      'Status LED D3 (yellow)
        PIN_D2          = 14                      'Status LED D2 (orange or red)
        PIN_D1          = 15                      'Status LED D1 (green)
        PIN_TRIGGER     = 29                      'Trigger Pulse Train / Select Frequency / I2C_SDA on startup
        PIN_TX          = 30                      'RS232 TX (Transmit) Pin
        PIN_RX          = 31                      'RS232 RX (Receive) Pin
        ''


        ''DIP Switches A and B, otherwise hardwired: Board Version
        ''
        ''  open = H, closed straight = L, closed angular = H
        ''
        ''  Board Version  |  SWDIP_B  |  SWDIP_A
        ''  ---------------|-----------|-----------
        ''             V1  |        L  |        L
        ''             V2  |        L  |        H
        ''             V3  |        H  |        L
        ''             V4  |        H  |        H
        ''
        ''  Switch SWDIP_B is tristate on board v1,
        ''  but will hopefully read low.
        ''
        SWDIP_A         = 16                      ''  SWDIP_A
        SWDIP_B         = 17                      ''  SWDIP_B
        ''


        ''DIP Switches 1..2: SPI Clock Driver
        ''
        ''  open = H, closed = L
        ''
        ''  SPI Clock Driver  |  SWDIP_2  |  SWDIP_1
        ''  ------------------|-----------|-----------
        ''               25%  |        H  |        H
        ''               50%  |        H  |        L
        ''               75%  |        L  |        H
        ''              100%  |        L  |        L
        ''
        SWDIP_1         = 18                      ''  SWDIP_1
        SWDIP_2         = 19                      ''  SWDIP_2
        ''


        ''DIP Switch 3: SPI Mode
        ''
        ''  open = H, closed = L
        ''
        ''  SPI Mode  |  SWDIP_3
        ''  ----------|-----------
        ''         0  |        H
        ''         3  |        L
        ''
        SWDIP_3         = 20                      ''  SWDIP_3
        ''


        ''DIP Switch 4: RS232 Baudrate
        ''
        ''  open = H, closed = L
        ''
        ''  RS232 Baudrate  |  SWDIP_4
        ''  ----------------|-----------
        ''           57600  |        H
        ''          115200  |        L
        ''
        SWDIP_4         = 21                      ''  SWDIP_4
        ''


        ''DIP Switch 5: Chip Register is volatile
        ''
        ''  open = H, closed = L
        ''
        ''  volatile?  |  SWDIP_5
        ''  -----------|-----------
        ''    unknown  |        H
        ''   volatile  |        L
        ''
        SWDIP_5         = 22                      ''  SWDIP_5
        ''


        ''DIP Switch 6: Power-up Type
        ''
        ''  open = H, closed = L
        ''
        ''  Power-up Type  |  SWDIP_6
        ''  ---------------|-----------
        ''           once  |        H
        ''      repetitve  |        L
        ''
        SWDIP_6         = 23                      ''  SWDIP_6
        ''


        ''Pins in respect to Board Version
        ''
        ''  Board v1:
        ''
        RST_DISABLE     = 16                      ''  RST_DISABLE       --- Inhibit signal pin on board v1 for net RS232_RST
        ''
        ''  Board v2:
        ''
        RS232_RST       = 24                      ''  RS232_RST         --- Sense RST input from host
        RST_INHIBIT     = 11                      ''  RST_INHIBIT       --- Inhibit signal pin on board v2 for net RS232_RST
        ADC_OUT         = 27                      ''  ADC_OUT           --- Sigma-delta ADC circuit, counter output
        ADC_CALIBRATION = 26                      ''  ADC_CALIBRATION   --- Sigma-delta ADC circuit, calibration pin, providing 0V/3.3V, tri-state when idle
        ADC_IN          = 25                      ''  ADC_IN            --- Sigma-delta ADC circuit, counter input
        ''


        ''Compound Bitmasks
        MASK_CEn_AVAIL      = (|<PIN_CEn0 | |<PIN_CEn1)                                                                 ''  MASK_CEn_AVAIL      --- bitmask for all available CEn pins
        MASK_SCLK_AVAIL     = (|<PIN_SCLK0 | |<PIN_SCLK1 | |<PIN_SCLK2 | |<PIN_SCLK3)                                   ''  MASK_SCLK_AVAIL     --- bitmask for all available SPI clock pins
        MASK_SPI_BUS_AVAIL  = (MASK_CEn_AVAIL | |<PIN_MOSI | |<PIN_WPn | |<PIN_HOLDn | MASK_SCLK_AVAIL | |<PIN_MISO)    ''  MASK_SPI_BUS_AVAIL  --- bitmask for pins that form the SPI Bus
        '
        'MASK_SCLK_ACTIVE:
        '
        'We can drive the SPI clock pin with different current setups, 4 pins
        'are available.
        '
        '* T500-8MB-Winbond requires one pin
        '* X60s-2MB-SST requires one pin
        '* X230-4MB-Macronix requires two pins
        '* X200-8MB-Winbond requires three pins
        '
        MASK_SCLK_ACTIVE100   = (|<PIN_SCLK0 | |<PIN_SCLK1 | |<PIN_SCLK2 | |<PIN_SCLK3)                                 ''  MASK_SCLK_ACTIVE100 --- bitmask for selected SPI clock pins in use: four
        MASK_SCLK_ACTIVE75    = (|<PIN_SCLK0 | |<PIN_SCLK1 | |<PIN_SCLK2)                                               ''  MASK_SCLK_ACTIVE75  --- bitmask for selected SPI clock pins in use: three
        MASK_SCLK_ACTIVE50    = (|<PIN_SCLK0 | |<PIN_SCLK1)                                                             ''  MASK_SCLK_ACTIVE50  --- bitmask for selected SPI clock pins in use: two
        MASK_SCLK_ACTIVE25    = (|<PIN_SCLK0)                                                                           ''  MASK_SCLK_ACTIVE25  --- bitmask for selected SPI clock pins in use: one
        MASK_SCLK_ACTIVE0     = 0                                                                                       ''  MASK_SCLK_ACTIVE0   --- bitmask for selected SPI clock pins in use: none
        MASK_SCLK_ACTIVE      = MASK_SCLK_ACTIVE100                                                                     ''  MASK_SCLK_ACTIVE    --- bitmask for selected SPI clock pins in use, not used by flashrom interface


{{


# Functions
}}


PUB     check_cable : r

        ''Check SPI Cable
        ''  FALSE = Cable not present
        ''  TRUE  = Cable present

        r := INA[PIN_PLUGTESTn] <> FALSE
        !r


PUB     get_board_config : r

        ''Retrieve board configuration from DIP switches.

        r := (!INA[SWDIP_6..SWDIP_1]) & %111111
        r <<= 2
        r |= INA[SWDIP_B..SWDIP_A] & %11


PUB     get_board_version (cfg)

        ''Get board version according to board configuration.

        RETURN (cfg & %00000011) + 1


PUB     get_pin_RSTinhibit (cfg)

        ''Which pin is used to inhibit net RS232_RST?

        CASE get_board_version(cfg)
            1:
                RETURN RST_DISABLE
            OTHER:
                RETURN RST_INHIBIT


PUB     get_monitor_hardware (cfg)

        ''Is SPI voltage monitor hardware available?

        CASE get_board_version(cfg)
            1:
                RETURN FALSE
            OTHER:
                RETURN TRUE


PUB     get_powerup_type (cfg)

        ''Check DIP switch 6:
        ''Returns TRUE for type repetitive, otherwise FALSE.

#   ifdef _CFG_PWRUP_ONCE
        RETURN FALSE
#   endif
#   ifdef _CFG_PWRUP_REPETITIVE
        RETURN TRUE
#   endif
#   ifdef _CFG_PWRUP_DIPSW
        CASE get_board_version(cfg)
            1:
                RETURN TRUE
            OTHER:
                RETURN (cfg & %10000000) <> FALSE
#   endif


PUB     get_suspend_inhibit (cfg)

        ''Check DIP switch 5:
        ''  Vcc_SPI Suspend allowed:    return FALSE
        ''  Vcc_SPI Suspend inhibited:  return TRUE

#   ifdef _CFG_SUSPEND_ALLOW
        RETURN FALSE
#   endif
#   ifdef _CFG_SUSPEND_INHIBIT
        RETURN TRUE
#   endif
#   ifdef _CFG_SUSPEND_DIPSW
        CASE get_board_version(cfg)
            1:
                RETURN FALSE
            OTHER:
                RETURN (cfg & %01000000) <> FALSE
#   endif


PUB     get_mode_SPI (cfg)

        ''Check DIP switch 3, set SPI mode

#   ifdef _CFG_SPIMODE0
        RETURN 0
#   endif
#   ifdef _CFG_SPIMODE3
        RETURN 3
#   endif
#   ifdef _CFG_SPIMODE_DIPSW
        CASE get_board_version(cfg)
            1:
                RETURN 3
            OTHER:
                IF (cfg & %00010000)
                    RETURN 0
                ELSE
                    RETURN 3
#   endif


PUB     get_driver_strength (cfg)

        ''Check DIP switches 1 and 2, set driver strength.

#   ifdef _CFG_DRV100
            RETURN %11
#   endif
#   ifdef _CFG_DRV75
            RETURN %10
#   endif
#   ifdef _CFG_DRV50
            RETURN %01
#   endif
#   ifdef _CFG_DRV25
            RETURN %00
#   endif
#   ifdef _CFG_DRV_DIPSW
        CASE get_board_version(cfg)
            1:
                RETURN %01
            OTHER:
                RETURN (cfg & %00001100) >> 2
#   endif


PUB     get_baudrate (cfg)

        ''Return baudrate according to DIP switch 4.

#   ifdef _CFG_BAUDRATE_115200
        CASE get_board_version(cfg)
            1:
                RETURN 115200
            OTHER:
                CASE (cfg & %00100000) >> 5
                    0:
                        RETURN 115200
                    1:
                        RETURN 57600
#   endif
#   ifdef _CFG_BAUDRATE_57600
        CASE get_board_version(cfg)
            1:
                RETURN 57600
            OTHER:
                CASE (cfg & %00100000) >> 5
                    0:
                        RETURN 115200
                    1:
                        RETURN 57600
#   endif
#   ifdef _CFG_BAUDRATE_38400
        RETURN 38400
#   endif


PUB     high (pin_number)

        ''Set pin high (1).
        ''Direction register bit is configured to output.

        OUTA[pin_number]~~
        DIRA[pin_number]~~


PUB     low (pin_number)

        ''Set pin low (0).
        ''Direction register bit is configured to output.

        OUTA[pin_number]~
        DIRA[pin_number]~~


{{


******************************************************************************
Terms of Use:

Zerocat Chipflasher is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

Zerocat Chipflasher is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with Zerocat Chipflasher.  If not, see <http://www.gnu.org/licenses/>.

************************************************************** End of File ***
}}
